package com.tzld.longarticle.recommend.server.service.recommend.rank;


import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.model.ConfigChangeEvent;
import com.ctrip.framework.apollo.spring.annotation.ApolloConfigChangeListener;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishPlanInputSourceTypesEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.RankStrategyEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ScoreStrategyEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.ServiceBeanFactory;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.config.StrategyIndexScoreWeightService;
import com.tzld.longarticle.recommend.server.service.recommend.score.AccountIndexReplacePoolConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService implements InitializingBean {

    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    private StrategyIndexScoreWeightService weightService;

    private static Map<String, Map<String, Integer>> staticStrategyPoolSourceTypeMap;

    @ApolloJsonValue("${strategyPoolSourceTypeConfig:{}}")
    private Map<String, Map<String, Integer>> strategyPoolSourceTypeMap;

    @Override
    public void afterPropertiesSet() throws Exception {
        RankService.staticStrategyPoolSourceTypeMap = strategyPoolSourceTypeMap;
    }

    @ApolloConfigChangeListener(interestedKeys = {"strategyPoolSourceTypeConfig"})
    public void configChange(ConfigChangeEvent changeEvent) {
        RankService.staticStrategyPoolSourceTypeMap = JSONObject.parseObject(changeEvent.getChange(
                "strategyPoolSourceTypeConfig").getNewValue(), Map.class);
        log.info("strategyPoolSourceTypeConfig change updateStaticValue success newValue:{}",
                JSONObject.toJSONString(staticStrategyPoolSourceTypeMap));
    }

    public static Integer getStrategyPoolSourceType(String strategy, Integer index) {
        Integer sourceType = 0;
        Map<String, Integer> indexSourceTypeMap = staticStrategyPoolSourceTypeMap.get(strategy);
        if (Objects.nonNull(indexSourceTypeMap)) {
            sourceType = indexSourceTypeMap.get(String.valueOf(index));
        }
        return sourceType;
    }

    public static Map<String, Integer> getStrategyPoolSourceType(String strategy) {
        return staticStrategyPoolSourceTypeMap.get(strategy);
    }

    public RankResult rank(RankParam param) {
        RankStrategy strategy = getRankStrategy(param);
        return strategy.rank(param);
    }

    private RankStrategy getRankStrategy(RankParam param) {
        RankStrategyEnum rankStrategyEnum = RankStrategyEnum.from(param.getStrategy());
        return (RankStrategy) ServiceBeanFactory.getBeanByName(rankStrategyEnum.getTaskExecutorName());
    }

    public static void printSortLog(String strategy, String accountName, String position, List<Content> contentList) {
        JSONArray jsonArray = new JSONArray();
        if (CollectionUtil.isEmpty(contentList)) {
            return;
        }
        contentList = contentList.subList(0, Math.min(contentList.size(), 200));
        for (Content content : contentList) {
            JSONObject obj = new JSONObject();
            obj.put("id", content.getId());
            obj.put("title", content.getTitle());
            obj.put("score", content.getScore());
            obj.put("scoreMap", content.getScoreMap());
            jsonArray.add(obj);
        }
        log.info("{} 账号名称 {} {}评分结果 {}", strategy, accountName, position, JSONObject.toJSONString(jsonArray));
    }

    public static void commonAddSecondContent(RankParam param, List<Content> result,
                                              String[] publishPool, String[] contentPools,
                                              Map<String, List<Content>> contentMap,
                                              Map<Integer, AccountIndexReplacePoolConfig> indexReplacePoolConfigMap,
                                              String strategy) {
        List<Content> pool = contentMap.get(contentPools[1]);
        if (CollectionUtils.isNotEmpty(pool)) {
            pool = contentSourceTypeFilter(param.getStrategy(), pool, 2);
        }
        RankService.printSortLog(strategy, param.getAccountName(), "次条", pool);
        if (CollectionUtils.isNotEmpty(pool)) {
            int i = RandomUtils.nextInt(0, Math.min(pool.size(), 5));
            int j = RandomUtils.nextInt(0, Math.min(pool.size(), 5));
            result.add(pool.get(i));
            // 替补 头条内容不足使用次条内容
            if (result.size() == 1 && pool.size() > 1) {
                while (i == j && pool.size() > 1) {
                    j = RandomUtils.nextInt(0, Math.min(pool.size(), 5));
                    if (i != j) {
                        publishPool[0] = contentPools[1];
                        result.add(pool.get(1));
                        break;
                    }
                }
            }
        } else {
            // 替补 根据设置替补内容池查找内容进行替补
            findReplacePoolContent(result, publishPool, contentMap, indexReplacePoolConfigMap);
        }
    }

    public static void findReplacePoolContent(List<Content> result, String[] publishPool,
                                              Map<String, List<Content>> contentMap,
                                              Map<Integer, AccountIndexReplacePoolConfig> indexReplacePoolConfigMap) {
        AccountIndexReplacePoolConfig replacePoolConfig = indexReplacePoolConfigMap.get(2);
        if (Objects.nonNull(replacePoolConfig)) {
            List<Content> poolReplace = contentMap.get(replacePoolConfig.getContentPool());
            if (CollectionUtils.isNotEmpty(poolReplace)) {
                publishPool[1] = replacePoolConfig.getContentPool();
                result.add(poolReplace.get(0));
            }
        }
    }

    public static List<Content> contentSourceTypeFilter(String strategy, List<Content> pool, Integer index) {
        Integer sourceType = getStrategyPoolSourceType(strategy, index);
        Integer videoSourceType = PublishPlanInputSourceTypesEnum.longArticleVideoPoolSource.getVal();
        List<Content> videoPool = pool.stream().filter(o -> Objects.equals(o.getSourceType(), videoSourceType)).collect(Collectors.toList());
        List<Content> otherPool = pool.stream().filter(o -> !Objects.equals(o.getSourceType(), videoSourceType)).collect(Collectors.toList());
        if (Objects.equals(sourceType, PublishPlanInputSourceTypesEnum.all.getVal())) {
            return pool;
        } else if (sourceType.equals(videoSourceType)) {
            if (CollectionUtils.isNotEmpty(videoPool)) {
                return videoPool;
            }
        }
        return otherPool;
    }

    public static void commonAdd38Content(RankParam param, List<Content> result, String[] contentPools,
                                          Map<String, List<Content>> contentMap, String strategy) {
        List<Content> pool = contentMap.get(contentPools[2]);
        if (CollectionUtils.isNotEmpty(pool)) {
            pool = contentSourceTypeFilter(param.getStrategy(), pool, 3);
        }
        if (CollectionUtils.isNotEmpty(pool) && param.getSize() > result.size()) {
            RankService.printSortLog(strategy, param.getAccountName(), "3-8", pool);
            result.addAll(pool.subList(0, Math.min(pool.size(), param.getSize() - result.size())));
        }
    }

    public List<RankItem> setRankItemScore(RankParam param) {
        List<Content> contents = param.getContents();
        String strategy = param.getStrategy();
        String ghId = param.getGhId();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        List<RankItem> result = new ArrayList<>();
        for (Content content : contents) {
            RankItem item = new RankItem();
            item.setContent(content);
            item.setScoreMap(content.getScoreMap());
            Double score = 0.0;
            int index = weightService.getIndex(item.getContent().getContentPoolType(), contentPools);
            switch (strategy) {
                case "ArticleRankV1":
                case "ArticleRankV2":
                case "ArticleRankV6":
                case "ArticleRankRandom":
                    break;
                case "ArticleRankV3":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())
                            || contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV4":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())
                            || contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE_CORRELATION.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV5":
                case "ArticleRankV15":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())
                            || contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV7":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_CORRELATION_RATE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV8":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_RATE.value());
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV9":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_RATE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV10":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_AVG_READ_SUM_RATE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankV11":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_RATE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_RATE.value())
                                + item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + (item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value()))
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    content.setHisPublishArticleList(null);
                    item.setScore(score);
                    break;
                case "ArticleRankV12":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_DE_WEIGHT_AVG_READ_SUM_RATE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.HIS_FISSION_DE_WEIGHT_AVG_READ_SUM_RATE.value())
                                + item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + (item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value()))
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    content.setHisPublishArticleList(null);
                    item.setScore(score);
                    break;
                case "ArticleRankV13":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        double similarityScore = Math.pow(item.getScore(ScoreStrategyEnum.SIMILARITY.value()),
                                weightService.getWeight(strategy, ghId, index,
                                        ScoreStrategyEnum.SIMILARITY.value()));
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_RATE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.HIS_FISSION_AVG_READ_RATE_RATE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                        score = score * similarityScore;
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + (item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value()))
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    content.setHisPublishArticleList(null);
                    item.setScore(score);
                    break;
                case "ArticleRankV14":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())) {
                        double similarityScore = Math.pow(item.getScore(ScoreStrategyEnum.SIMILARITY.value()),
                                weightService.getWeight(strategy, ghId, index,
                                        ScoreStrategyEnum.SIMILARITY.value()));
                        score = item.getScore(ScoreStrategyEnum.HIS_FISSION_DE_WEIGHT_AVG_READ_SUM_RATE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.HIS_FISSION_DE_WEIGHT_AVG_READ_SUM_RATE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                        score = score * similarityScore;
                    } else if (contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = (item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.SIMILARITY.value()))
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + (item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value()))
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    content.setHisPublishArticleList(null);
                    item.setScore(score);
                    break;
                case "ArticleRankHisJump":
                    if (contentPools[0].equals(item.getContent().getContentPoolType())
                            || contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value())
                                    * weightService.getWeight(strategy, ghId, index,
                                    ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankInfinite":
                case "ArticleRankLate":
                    if (contentPools[2].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                * weightService.getWeight(strategy, ghId, index,
                                ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
                case "ArticleRankFWH":
                    content.setHisPublishArticleList(null);
                    if (contentPools[0].equals(item.getContent().getContentPoolType())
                            || contentPools[1].equals(item.getContent().getContentPoolType())) {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                        if (item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value()) >= 0) {
                            score += item.getScore(ScoreStrategyEnum.VIEW_COUNT_RATE.value());
                        }
                    } else {
                        score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                                + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                                + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                                + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                                + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                                + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
                    }
                    content.setScore(score);
                    item.setScore(score);
                    break;
            }
            result.add(item);
        }
        result.sort((o1, o2) -> -Double.compare(o1.getScore(), o2.getScore()));
        param.setRankItems(result);
        return result;
    }

    private void setSafeScoreDecrease(RankParam param) {

    }
}
