package com.tzld.longarticle.recommend.server.service.recommend.rank.strategy;

import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.rank.*;
import com.tzld.longarticle.recommend.server.service.recommend.score.AccountIndexReplacePoolConfig;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@Slf4j
public class HisJumpRankStrategy implements RankStrategy {

    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {
        List<Content> result = new ArrayList<>();

        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        Map<Integer, AccountIndexReplacePoolConfig> indexReplacePoolConfigMap = accountContentPoolConfigService.getContentReplacePools(param.getAccountName());

        List<RankItem> items = param.getRankItems();
        
        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);

        // 3 文章按照内容池分组
        Map<String, List<Content>> contentMap = new HashMap<>();
        for (Content c : contents) {
            List<Content> data = contentMap.computeIfAbsent(c.getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }
        // 4 选文章
        String[] publishPool = Arrays.copyOf(contentPools, contentPools.length);

        // 头
        List<Content> pool1 = contentMap.get(contentPools[0]);
        RankService.printSortLog(param.getStrategy(), param.getAccountName(), "头条", pool1);
        if (CollectionUtils.isNotEmpty(pool1)) {
            int i = RandomUtils.nextInt(0, Math.min(pool1.size(), 20));
            result.add(pool1.get(i));
        } else {
            RankStrategy.sendFeishuFirstPoolEmpty(param, contentPools[0]);
            return new RankResult(result);
        }
        // 次
        List<Content> pool2 = contentMap.get(contentPools[1]);
        RankService.printSortLog(param.getStrategy(), param.getAccountName(), "次条", pool2);
        if (CollectionUtils.isNotEmpty(pool2)) {
            int i = RandomUtils.nextInt(0, Math.min(pool2.size(), 20));
            result.add(pool2.get(i));
        } else {
            // 替补 根据设置替补内容池查找内容尽心替补
            AccountIndexReplacePoolConfig replacePoolConfig = indexReplacePoolConfigMap.get(2);
            if (Objects.nonNull(replacePoolConfig)) {
                List<Content> pool2Replace = contentMap.get(replacePoolConfig.getContentPool());
                if (CollectionUtils.isNotEmpty(pool2Replace)) {
                    publishPool[1] = replacePoolConfig.getContentPool();
                    int i = RandomUtils.nextInt(0, Math.min(pool2Replace.size(), 20));
                    result.add(pool2Replace.get(i));
                }
            }
        }

        // 3-8
        List<Content> pool = contentMap.get(contentPools[2]);
        RankService.printSortLog(param.getStrategy(), param.getAccountName(), "3-8", pool);
        if (CollectionUtils.isNotEmpty(pool) && param.getSize() > result.size()) {
            int slotNum = param.getSize() - result.size();
            List<Content> subPool = pool.subList(0, Math.min(pool.size(), 30));
            Collections.shuffle(subPool);
            result.addAll(subPool.subList(0, Math.min(subPool.size(), slotNum)));
        }

        RankStrategy.deduplication(result, contentMap, publishPool);

        return new RankResult(result);
    }

}
