package com.tzld.longarticle.recommend.server.service.recommend.rank.strategy;

import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.rank.*;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreParam;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class LateRankStrategy implements RankStrategy {

    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {
        List<Content> result = new ArrayList<>();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        ScoreParam scoreParam = RankStrategy.convertToScoreParam(param);
        scoreParam.setContents(scoreParam.getContents().stream().filter(
                c -> contentPools[2].equals(c.getContentPoolType())).collect(Collectors.toList()));

        List<RankItem> items = param.getRankItems();
        
        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);

        // 3 文章按照内容池分组
        Map<String, List<Content>> contentMap = new HashMap<>();
        for (Content c : contents) {
            List<Content> data = contentMap.computeIfAbsent(c.getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }

        // 全部使用3-8内容池
        RankService.commonAdd38Content(param, result, contentPools, contentMap, param.getStrategy());

        return new RankResult(result);
    }

}
