package com.tzld.longarticle.recommend.server.service.recommend.rank.strategy;


import com.tzld.longarticle.recommend.server.common.enums.recommend.ScoreStrategyEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.rank.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankV2Strategy implements RankStrategy {

    public RankResult rank(RankParam param) {

        List<RankItem> items = param.getRankItems();

        // 1 排序
        Collections.sort(items, (o1, o2) -> {
            int result = -Double.compare(
                    o1.getScore(ScoreStrategyEnum.SIMILARITY.value()),
                    o2.getScore(ScoreStrategyEnum.SIMILARITY.value()));
            if (result != 0) {
                return result;
            }

            return -Double.compare(
                    o1.getScore(ScoreStrategyEnum.VIEW_COUNT.value()),
                    o2.getScore(ScoreStrategyEnum.VIEW_COUNT.value()));
        });
        // 2 选文章
        List<Content> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(items)) {
            for (int i = 0; i < Math.min(items.size(), param.getSize()); i++) {
                result.add(items.get(i).getContent());
            }
        }

        return new RankResult(result);
    }

}
