package com.tzld.longarticle.recommend.server.service.recommend.rank.strategy;


import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.rank.*;
import com.tzld.longarticle.recommend.server.service.recommend.score.AccountIndexReplacePoolConfig;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankV5Strategy implements RankStrategy {

    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    private ArticleRepository articleRepository;

    public RankResult rank(RankParam param) {
        List<Content> result = new ArrayList<>();

        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        Map<Integer, AccountIndexReplacePoolConfig> indexReplacePoolConfigMap = accountContentPoolConfigService.getContentReplacePools(param.getAccountName());

        List<RankItem> items = param.getRankItems();

        // 相似度评分为0 报警返回
        List<Article> hisPublishFirstArticleList = articleRepository.getByGhIdAndItemIndexAndTypeEqualsAndStatusEquals(
                param.getGhId(), 1, param.getType(), 1);
        if (RankStrategy.SimilarityScoreZero(items, param, hisPublishFirstArticleList)) {
            return new RankResult(result);
        }

        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);

        // 3 文章按照内容池分组
        Map<String, List<Content>> contentMap = new HashMap<>();
        for (Content c : contents) {
            List<Content> data = contentMap.computeIfAbsent(c.getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }
        // 4 选文章
        String[] publishPool = Arrays.copyOf(contentPools, contentPools.length);

        // 头
        List<Content> pool1 = contentMap.get(contentPools[0]);
        RankService.printSortLog(param.getStrategy(), param.getAccountName(), "头条", pool1);
        if (CollectionUtils.isNotEmpty(pool1)) {
            result.add(pool1.get(0));
        } else {
            RankStrategy.sendFeishuFirstPoolEmpty(param, contentPools[0]);
            return new RankResult(result);
        }
        // 次
        RankService.commonAddSecondContent(param, result, publishPool, contentPools, contentMap,
                indexReplacePoolConfigMap, param.getStrategy());

        // 3-8
        RankService.commonAdd38Content(param, result, contentPools, contentMap, param.getStrategy());

        RankStrategy.deduplication(result, contentMap, publishPool);

        return new RankResult(result);
    }

}
